@@ -1,7 +1,7 @@
 ;;; exwm-systemtray.el --- System Tray Module for  -*- lexical-binding: t -*-
 ;;;                        EXWM
 
-;; Copyright (C) 2016-2023 Free Software Foundation, Inc.
+;; Copyright (C) 2016-2024 Free Software Foundation, Inc.
 
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
 
@@ -24,9 +24,9 @@
 
 ;; This module adds system tray support for EXWM.
 
-;; To use this module, load and enable it as follows:
-;;   (require 'exwm-systemtray)
-;;   (exwm-systemtray-enable)
+;; To use this module, enable it as follows:
+;;
+;;   (exwm-systemtray-mode 1)
 
 ;;; Code:
 
@@ -46,20 +46,23 @@
    (visible :initarg :visible))
   :documentation "Attributes of a system tray icon.")
 
-(defclass xcb:systemtray:-ClientMessage
-  (xcb:icccm:--ClientMessage xcb:ClientMessage)
-  ((format :initform 32)
-   (type :initform 'xcb:Atom:MANAGER)
-   (time :initarg :time :type xcb:TIMESTAMP)      ;new slot
-   (selection :initarg :selection :type xcb:ATOM) ;new slot
-   (owner :initarg :owner :type xcb:WINDOW))      ;new slot
-  :documentation "A systemtray client message.")
-
 (defgroup exwm-systemtray nil
   "System tray."
-  :version "25.3"
   :group 'exwm)
 
+;;;###autoload
+(define-minor-mode exwm-systemtray-mode
+  "Toggle EXWM systemtray support."
+  :global t
+  :group 'exwm-systemtray
+  (exwm--global-minor-mode-body systemtray))
+
+(defun exwm-systemtray-enable ()
+  "Enable EXWM systemtray support."
+  (message "`exwm-systemtray-enable' is obsolete. Use `exwm-systemtray-mode' instead.")
+  (exwm-systemtray-mode 1))
+(make-obsolete 'exwm-systemtray-enable "Use `exwm-systemtray-mode' instead." "0.30")
+
 (defcustom exwm-systemtray-height nil
   "System tray height.
 
@@ -97,7 +100,8 @@ TrueColor-24\" can be used to force Emacs to use 24-bit depth."
 using 32-bit depth.  Using `workspace-background' instead.")
            (setq value 'workspace-background))
          (set-default symbol value)
-         (when (and exwm-systemtray--connection
+         (when (and exwm-systemtray-mode
+                    exwm-systemtray--connection
                     exwm-systemtray--embedder-window)
            ;; Change the background color for embedder.
            (exwm-systemtray--set-background-color)
@@ -488,7 +492,9 @@ Argument DATA contains the raw event data."
 (cl-defun exwm-systemtray--init ()
   "Initialize system tray module."
   (exwm--log)
-  (cl-assert (not exwm-systemtray--connection))
+  ;; idempotent initialization
+  (when exwm-systemtray--connection
+    (cl-return-from exwm-systemtray--init))
   (cl-assert (not exwm-systemtray--list))
   (cl-assert (not exwm-systemtray--selection-owner-window))
   (cl-assert (not exwm-systemtray--embedder-window))
@@ -543,7 +549,7 @@ Argument DATA contains the raw event data."
                        :destination exwm--root
                        :event-mask xcb:EventMask:StructureNotify
                        :event (xcb:marshal
-                               (make-instance 'xcb:systemtray:-ClientMessage
+                               (make-instance 'xcb:icccm:-ManagerSelection
                                               :window exwm--root
                                               :time xcb:Time:CurrentTime
                                               :selection
@@ -689,12 +695,6 @@ Argument DATA contains the raw event data."
     (when (boundp 'exwm-randr-refresh-hook)
       (remove-hook 'exwm-randr-refresh-hook #'exwm-systemtray--refresh-all))))
 
-(defun exwm-systemtray-enable ()
-  "Enable system tray support for EXWM."
-  (exwm--log)
-  (add-hook 'exwm-init-hook #'exwm-systemtray--init)
-  (add-hook 'exwm-exit-hook #'exwm-systemtray--exit))
-
 
 
 (provide 'exwm-systemtray)
